from WidgetFinder import NodeWidgetFinder
from EventFinder import EventFinder

class EventManager:
    """Mixin for ZWidgetManager dealing with events.
    """
    
    def _key(self, document, klass):
        """Create a session key for a document and class.
        FIXME: this would cause a bug if two classes have the same name.
        """
        return klass.__name__ + document._p_oid
        
    def _get(self, document, klass):
        """Get an object of a certain class for a certain document.
        """
        if not hasattr(self, 'FSSession'):
            raise Error, "Event system for XMLWidgets requires FSSession."
    
        session = self.FSSession
        
        key = self._key(document, klass)

        if session.has_key(key):
            return session[key]
        else:
            obj = klass()
            session.set(key, obj)
            return obj

    def _set(self, document, obj):
        if not hasattr(self, 'FSSession'):
            raise Error, "Event system for XMLWidgets requires FSSession."

        session = self.FSSession

        key = self._key(document, obj.__class__)
    
        session.set(key, obj)

    def addNodeWidget(self, widget_id, node):
        """Add a node widget to the nodewidgetfinder.

        widget_id -- the id of the widget that will handle node
        node -- the node to use the special widget for
        """
        # get owner document of node
        document = node.getOwnerDocument()
        # get nodewidgetfinder
        nodewidgetfinder = self._get(document, NodeWidgetFinder)

        # add node widget
        widget = getattr(self, widget_id)
        nodewidgetfinder.addWidget(widget, node)

        # store session data again
        self._set(document, nodewidgetfinder)

    def clearNodeWidgets(self, document):
        """Clear all node-specific widgets for document from session.
        """
        # get information from session
        nodewidgetfinder = self._get(document, NodeWidgetFinder)

        # add node widget
        nodewidgetfinder.clear()

        # store session data again
        self._set(document, nodewidgetfinder)
        
    def _findNodeWidgetId(self, node):
        """Look up if node occurs in NodeWidgetFinder for this document.

        Result: widget_id if widget found, or None if none found
        """
        nodewidgetfinder = self._get(node.getOwnerDocument(),
                                     NodeWidgetFinder)
        return nodewidgetfinder.find(node)

    def _event_id(self, node, widget, methodname):
        """Get event_id for widget's method on node.

        node -- the node to add the event for
        widget -- the widget that will handle the event
        methodname -- the name of the event (on the widget)
        """
        # get owner document of node
        document = node.getOwnerDocument()
        # get eventfinder from session
        eventfinder = self._get(document, EventFinder)

        # add event to eventfinder
        event_id = eventfinder.addEvent(node, widget, methodname)

        # store eventfinder in session again
        self._set(document, eventfinder)

        # return event_id
        return event_id
    
    def event_url(self, node, widget, methodname):
        """Create URL for event handler.

        node -- the node to add the event for
        widget -- the widget that will handle the event
        methodname -- the name of the event (on the widget)
        """
        event_id = self._event_id(node, widget, methodname)
        
        # now return URL
        # use viewMethod if it exists, otherwise use current folder
        if self.viewMethod:
            return (self.viewMethod +
                    "?event_id=%s" % event_id)
        else:
            return (".?event_id=%s" % event_id)

    def event_form_url(self):
        """Create URL for event handler attached to form.
        """
        if self.viewMethod:
            return self.viewMethod
        else:
            return "."

    def event_form_hidden(self, node, widget, methodname):
        """Create hidden field for form with event_id.
        """
        event_id = self._event_id(node, widget, methodname)

        return '<input type="hidden" name="event_id" value="%s">' % event_id
    
    def clearEvents(self, document):
        """Clear events for document from eventfinder.
        """    
        eventfinder = self._get(document, EventFinder)
        
        eventfinder.clear()
    
        self._set(document, eventfinder)

    def handleEvent(self, document, event_id, REQUEST):
        """Handle incoming event.

        document -- the document for which the event is arriving
        event_id -- the id of the event
        REQUEST -- web request object
        """
        eventfinder = self._get(document, EventFinder)
  
        # look up event
        node_url, widget_id, methodname = eventfinder.find(int(event_id))

        # find node associated with url
        node = REQUEST.resolve_url(node_url)
        
        # get appropriate widget
        widget = getattr(self, widget_id)
    
        # execute methodname on widget
        return getattr(widget, methodname)(node, REQUEST)

class Error(Exception):
    pass
